-----------------------------------------------------------------------------------------------
-- Client Lua Script for NexusTime
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- NexusTime Module Definition
-----------------------------------------------------------------------------------------------
local NexusTime = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

local SecondsToggle
local TimeFormatToggle
local top, right, left, bottom
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function NexusTime:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	SecondsToggle = false
	TimeFormatToggle = true

    return o
end

function NexusTime:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- NexusTime OnLoad
-----------------------------------------------------------------------------------------------
function NexusTime:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("NexusTime.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- NexusTime OnDocLoaded
-----------------------------------------------------------------------------------------------
function NexusTime:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "NexusTimeForm", nil, self)
		self.nexusTimeLabel = Apollo.LoadForm(self.xmlDoc, "NexusTimeLabel", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
		self.nexusTimeLabel:Show(true, true)
		
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("nexustime", "OnNexusTimeOn", self)

		self.timer = ApolloTimer.Create(0.14, true, "OnTimer", self)

		-- Do additional Addon initialization here
		if left and top and right and bottom then
			self.nexusTimeLabel:SetAnchorOffsets(left, top, right, bottom)
		end
			
		self:DisplayTime()
	end
end

function NexusTime:OnSave(eLevel)

    -- create a table to hold our data
    local tSave = {}

	tSave.secondsToggle = SecondsToggle
	tSave.timeFormatToggle = TimeFormatToggle
	tSave.left, tSave.top, tSave.right, tSave.bottom = self.nexusTimeLabel:GetAnchorOffsets()

    -- simply return this value and Apollo will save the file!
    return tSave
end

function NexusTime:OnRestore(eLevel, tData)
	SecondsToggle = tData.secondsToggle
	TimeFormatToggle = tData.timeFormatToggle

	top = tData.top
	right = tData.right
	bottom = tData.bottom
	left = tData.left	
end

-----------------------------------------------------------------------------------------------
-- NexusTime Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/nexustime"
function NexusTime:OnNexusTimeOn()

	if not SecondsToggle then
		self.wndMain:FindChild("SecondsToggleButton"):SetText("Off")
	end
	
	if not TimeFormatToggle then
		self.wndMain:FindChild("TimeFormatButton"):SetText("24 hr")
	end

	self.wndMain:Invoke() -- show the window
end

function NexusTime:OnTimer()	
	self:DisplayTime()	
end

function NexusTime:DisplayTime()
	local displayText
	local period = "AM"
	
	local worldTime = GameLib.GetWorldTimeOfDay()
	local hours = math.floor(worldTime/3600)
	local minutes = math.floor(worldTime/60 - (hours*60))
	local seconds = math.floor(worldTime - hours*3600 - minutes*60)
	
	-- Find modulus hours for 12 hr format if it is toggled
	if TimeFormatToggle and hours ~= 0 and hours ~= 12 then
		hours = math.floor((worldTime/3600) % 12)
	end
	
	displayText = string.format("%02.f:%02.f", hours, minutes)
	
	-- Concatenate seconds to display text if it is toggled
	if SecondsToggle then
		displayText = string.format("%s:%02.f", displayText, seconds)
	end	
	
	-- Figure out period and concatenate to display text
	-- if it is toggled
	if TimeFormatToggle then
		if math.floor(worldTime/3600) >= 12 then
     		period = "PM"
		end
		displayText = string.format("%s %s", displayText, period)
	end
		
	self.nexusTimeLabel:SetText(displayText)	
end

-----------------------------------------------------------------------------------------------
-- NexusTimeForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function NexusTime:OnOK()
	self.wndMain:Close() -- hide the window
end

function NexusTime:OnSecondsToggle( wndHandler, wndControl, eMouseButton )	
	local button = self.wndMain:FindChild("SecondsToggleButton")
	SecondsToggle = not SecondsToggle
	
	if SecondsToggle then
		button:SetText("On")
	else
		button:SetText("Off")
	end
	
end

function NexusTime:OnTimeFormatButton( wndHandler, wndControl, eMouseButton )
	local button = self.wndMain:FindChild("TimeFormatButton")
	TimeFormatToggle = not TimeFormatToggle
	
	if TimeFormatToggle then
		button:SetText("12 hr")
	else
		button:SetText("24 hr")
	end
end

-----------------------------------------------------------------------------------------------
-- NexusTime Instance
-----------------------------------------------------------------------------------------------
local NexusTimeInst = NexusTime:new()
NexusTimeInst:Init()
